cordova-plugin-ws1intelligence
==============================

This plugin enables access to Workspace ONE Intelligence SDK features for Cordova applications for iOS and Android.

License
--------
By integrating or downloading the software development kit (SDK) you accept the [VMware License](https://vdc-download.vmware.com/vmwb-repository/dcr-public/77d50615-6a61-4704-be18-30e8d9bac6ac/75789b6e-1ae4-4716-ab46-c6ce48b155fe/VMware%20Workspace%20ONE%20Intelligence%20SDK%20Software%20Development%20Kit%20License%20Agreement.pdf)

Installation
------------

In your application directory, simply type `cordova-plugin-ws1intelligence` at the command line to install the plugin.

Initialization
--------------

The Workspace ONE Intelligence Cordova plugin must be initialized by the host application prior to use. To do so, you must call an initialization function with your valid iOS and Android app ID.

WS1Intelligence.init({ 'iosAppID' : 'YOUR_IOS_APP_ID_HERE',
                       'androidAppID' : 'YOUR_ANDROID_APP_ID_HERE'});

This initialization call can be made in the onDeviceReady function.

You should see subsequent messages in your application logs that confirms successful initialization.

Support
-------

* iOS 12.0+ / Xcode 12.2
* Android API 16 and above

Functions
---------

The Workspace ONE Intelligence Cordova Plugin exposes the native interface of both the Android and iOS SDKs to Cordova applications through a wrapper interface.

<dl>

### Functions available for Android and iOS

<dt><code>init(appID)</code></dt>
<dd style="padding-bottom: 8pt">Initializes the Workspace ONE Intelligence SDK.</dd>

<dt><code>setLoggingLevel(loggingLevel)</code></dt>
<dd style="padding-bottom: 8pt">Sets the logging level of debug logs for the WS1 Intelligence SDK.</dd>

Logging Levels

* WS1IntelligenceLoggingLevelSilent
* WS1IntelligenceLoggingLevelError
* WS1IntelligenceLoggingLevelWarning
* WS1IntelligenceLoggingLevelInfo
* WS1IntelligenceLoggingLevelDebug
* WS1IntelligenceLoggingLevelVerbose

<dt><code>leaveBreadcrumb(breadCrumb)</code></dt>
<dd style="padding-bottom: 8pt">Logs a breadcrumb. Parameter is a string.</dd>

<dt><code>setAsyncBreadcrumbMode(asyncBreadcrumbsEnabled)</code></dt>
<dd style="padding-bottom: 8pt">Toggles asynchronous breadcrumb logging mode. Parameter is a boolean value. By default, breadcrumbs are flushed to disk synchronously to help ensure an accurate record prior to a potential crash. For performance reasons, an application may prefer to use asynchronous logging.</dd>

<dt><code>beginUserFlow(userFlowName)</code></dt>
<dd style="padding-bottom: 8pt">Starts a user flow with the provided name. Parameter is a string.</dd>

<dt><code>endUserFlow(userFlowName)</code></dt>
<dd style="padding-bottom: 8pt">Ends successfully a user flow with the provided name. Parameter is a string.</dd>

<dt><code>failUserFlow(userFlowName)</code></dt>
<dd style="padding-bottom: 8pt">Fails a user flow with the provided name. Parameter is a string.</dd>

<dt><code>cancelUserFlow(userFlowName)</code></dt>
<dd style="padding-bottom: 8pt">Cancels a user flow with the provided name. Parameter is a string.</dd>

<dt><code>logHandledException(exception)</code></dt>
<dd style="padding-bottom: 8pt">Logs a handled exception. Parameter is an exception. Use this method when a Javascript exception is caught in code.</dd>

<dt><code>logUnhandledException(exception)</code></dt>
<dd style="padding-bottom: 8pt">Logs an unhandled exception. Parameter is an exception. This method is used for exceptions that were not caught in application code.</dd>

<dt><code>logNetworkRequest(method, url, responseTime, bytesRead, bytesSent, responseCode, error)</code></dt>
<dd style="padding-bottom: 8pt">Logs details of a network request transaction.</dd>

<dt><code>logLocation(latitude, longitude)</code></dt>
<dd style="padding-bottom: 8pt">Logs current device location</dd>

<dt><code>setOptOutStatus(status)</code></dt>
<dd style="padding-bottom: 8pt">Enable or disable opt out, which prevents any data from being logged.</dd>

<dt><code>getOptOutStatus(callback)</code></dt>
<dd style="padding-bottom: 8pt">Get the current opt-out status. Status is returned via a callback provided by the caller.</dd>

<dt><code>getUserUUID(callback)</code></dt>
<dd style="padding-bottom: 8pt">Get the Workspace ONE Intelligence UUID for the current device. UUID is returned via a callback provided by the caller.(iOS only)</dd>

<dt><code>didCrashOnLastLoad(callback)</code></dt>
<dd style="padding-bottom: 8pt">Checks if the previous session ended in a crashed state. Crash information is returned via a callback provided by the caller.</dd>

<dt><code>setUsername(username)</code></dt>
<dd style="padding-bottom: 8pt">Sets a username which is affiliated with all following events. Parameter is a string.</dd>

</dl>

