// Copyright 2021 VMWare

var WS1_INTELLIGENCE_VERSION = "v1.0.0";

var exec = require("cordova/exec");

// You can add your custom callbacks in the functions below
function success () {}
function fail () {}

var	WS1Intelligence = {

    // Initialization

    init: function(appID) {
        cordova.exec(success, fail, "CDVWS1Intelligence", "ws1IntelligenceInit", [appID]);
        return this;
    },

    setLoggingLevel: function(loggingLevel) {
        cordova.exec(success, fail, "CDVWS1Intelligence", "ws1IntelligenceSetLoggingLevel", [loggingLevel]);
        return this;
    },

    // Breadcrumb

    leaveBreadcrumb: function(breadCrumb) {
        cordova.exec(success, fail, "CDVWS1Intelligence", "ws1IntelligenceLeaveBreadcrumb", [breadCrumb]);
        return this;
    },

    setAsyncBreadcrumbMode: function(asyncBreadcrumbsEnabled) {
        cordova.exec(success, fail, "CDVWS1Intelligence", "ws1IntelligenceSetAsyncBreadcrumbMode", [asyncBreadcrumbsEnabled]);
        return this;
    },

    // User Flow

    beginUserFlow: function(userFlowName) {
        cordova.exec(success, fail, "CDVWS1Intelligence", "ws1IntelligenceBeginUserFlow", [userFlowName]);
        return this;
    },

    endUserFlow: function(userFlowName) {
        cordova.exec(success, fail, "CDVWS1Intelligence", "ws1IntelligenceEndUserFlow", [userFlowName]);
        return this;
    },

    failUserFlow: function(userFlowName) {
        cordova.exec(success, fail, "CDVWS1Intelligence", "ws1IntelligenceFailUserFlow", [userFlowName]);
        return this;
    },

    cancelUserFlow: function(userFlowName) {
        cordova.exec(success, fail, "CDVWS1Intelligence", "ws1IntelligenceCancelUserFlow", [userFlowName]);
        return this;
    },

    // Exceptions

    logHandledException: function(exception) {
        cordova.exec(success, fail, "CDVWS1Intelligence", "ws1IntelligenceLogHandledException", [exception.name, exception.message, exception.stack]);
        return this;
    },

    logUnhandledException: function(exception) {
        cordova.exec(success, fail, "CDVWS1Intelligence", "ws1IntelligenceLogUnhandledException", [exception.name, exception.message, exception.stack]);
        return this;
    },

    // Network Requests

    logNetworkRequest: function(method, url, responseTime, bytesRead, bytesSent, responseCode, error) {
        cordova.exec(success, fail, "CDVWS1Intelligence", "ws1IntelligenceLogNetworkRequest", [method, url, responseTime, bytesRead, bytesSent, responseCode, error]);
        return this;
    },

    // Location

    logLocation: function(latitude, longitude) {
        cordova.exec(success, fail, "CDVWS1Intelligence", "ws1IntelligenceLogLocation", [latitude, longitude]);
        return this;
    },

    // Opt Out Status

    setOptOutStatus: function(status) {
        cordova.exec(success, fail, "CDVWS1Intelligence", "ws1IntelligenceSetOptOutStatus", [status]);
        return this;
    },

    getOptOutStatus: function(callback) {

        cordova.exec(function (result) {
            if (callback && typeof(callback) == "function") {
                try {
                    callback(result);
                } catch (exception) {
                    // Not handled
                }
            }
        }, function () {
            console.log("WS1 Intelligence Error: Could not get opt out status");
        }, "CDVWS1Intelligence", "ws1IntelligenceGetOptOutStatus");
    },

    getUserUUID: function(callback) {

        cordova.exec(function (result) {
            if (callback && typeof(callback) == "function") {
                try {
                    callback(result);
                } catch (exception) {
                    // Not handled
                }
            }
        }, function () {
            console.log("WS1 Intelligence Error: Could not get user UUID");
        }, "CDVWS1Intelligence", "ws1IntelligenceGetUserUUID");
    },

    didCrashOnLastLoad: function(callback) {

        cordova.exec(function (result) {
            if (callback && typeof(callback) == "function") {
                try {
                    callback(result);
                } catch (exception) {
                    // Not handled
                }
            }
        }, function () {
            console.log("WS1 Intelligence Error: Could not get opt out status");
        }, "CDVWS1Intelligence", "ws1IntelligenceDidCrashOnLastLoad");
    },

    // Username

    setUsername: function(username) {
        cordova.exec(success, fail, "CDVWS1Intelligence", "ws1IntelligenceSetUsername", [username]);
        return this;
    },
};

console.log("Using WS1 Intelligence Plugin " + WS1_INTELLIGENCE_VERSION);

module.exports = WS1Intelligence;
